Imports System.IO
Imports System.Collections
Imports System.Windows.Forms
Imports System.Drawing

Public Class Listview
    Implements IDisposable 'Inherit from IDisposable if you need Basic4ppc to free resources when the application is closed.

    Private WithEvents ListView1 As System.Windows.Forms.ListView 'The "real" object.
    Public Event Click As EventHandler 'The event handler name must match the event name.
    Private eventClickObject() As Object 'One object[] for each event (one event here).
    Private selectedRow As Integer

    Sub New(ByVal FormName As Control, ByVal Left As Int32, ByVal Top As Int32, ByVal Width As Int32, ByVal Height As Int32)
        ListView1 = New System.Windows.Forms.ListView
        With ListView1
            .View = View.Details
            .Left = Left
            .Top = Top
            .Width = Width
            .Height = Height
        End With

        FormName.Controls.Add(ListView1)

        'This is the object that will pass with the event.
        'It must be in that format exactly.
        eventClickObject = New Object() {Me, "Click"}
    End Sub

    Private Sub ListView1_ColumnClick(ByVal sender As Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) Handles ListView1.ColumnClick

    End Sub

    Private Sub ListView1_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListView1.Click
        Dim lItm As ListViewItem
        For Each lItm In ListView1.Items
            If lItm.Selected Then
                Exit For
            End If
        Next
        selectedRow = lItm.Index
        RaiseEvent Click(eventClickObject, Nothing)
    End Sub

    Public Sub AddColumns(ByVal Text As String, Optional ByVal ColumnWidth As Int32 = -2, Optional ByVal TextAlignment As Int32 = 0)
        If TextAlignment = 0 Then
            ListView1.Columns.Add(Text, ColumnWidth, HorizontalAlignment.Left)
        ElseIf TextAlignment = 1 Then
            ListView1.Columns.Add(Text, ColumnWidth, HorizontalAlignment.Right)
        ElseIf TextAlignment = 2 Then
            ListView1.Columns.Add(Text, ColumnWidth, HorizontalAlignment.Center)
        End If
    End Sub

    Public Sub Additem(ByVal ParamArray Celle() As String)
        Dim i As Integer = 0
        Dim n As Integer = 0

        i = UBound(Celle)

        If i > 0 Then
            With ListView1.Items.Add(New ListViewItem(Celle(0)))
                For n = 1 To ListView1.Columns.Count - 1
                    If n <= i Then
                        .SubItems.Add(Celle(n))
                    End If
                Next n
            End With
        End If
        ZeileEinfarben()
    End Sub

    Public Sub RemoveItem(ByVal Index As Integer)
        Dim n As Integer
        For n = 0 To ListView1.Items.Count
            If n = Index Then
                ListView1.Items.RemoveAt(n)
                Exit For
            End If
        Next
    End Sub

    Public Sub LoadCSV(ByVal Datei As String, ByVal Separator As Char, ByVal HeaderExist As Boolean, ByVal CreateColumns As Boolean)
        Dim objReader As New StreamReader(Datei)
        Dim sLine As String = ""
        Dim strArray() As String ' Das Array fr die einzelnen cellen
        Dim tmpArray() As String ' Ein Array fr zwischenspeicher
        Dim i, n As Int32
        Dim pos As Int32
        Dim hochkomma As Char = """"

        'Erste Zeile mit Header berspringen
        If HeaderExist = True Then
            sLine = objReader.ReadLine()

            'Columns erzeugen
            If CreateColumns = True Then
                tmpArray = sLine.Split(Separator) ' In ein String-Array konvertieren
                For i = 0 To UBound(tmpArray)
                    AddColumns(tmpArray(i), 50, 0)
                Next
            End If
        End If

        Do
            sLine = objReader.ReadLine()
            If Not sLine Is Nothing Then
                tmpArray = sLine.Split(Separator) ' In ein String-Array konvertieren
                ReDim strArray(UBound(tmpArray))
                i = 0
                n = 0
                Do
                    pos = InStr(tmpArray(i), hochkomma, CompareMethod.Text)
                    If pos = 0 Then
                        strArray(n) = tmpArray(i)
                    Else
                        strArray(n) = Replace(tmpArray(i) & Separator & tmpArray(i + 1), hochkomma, "")
                        i = i + 1
                    End If
                    n = n + 1
                    i = i + 1
                Loop Until i > UBound(tmpArray)
                Additem(strArray)
            End If
        Loop Until sLine Is Nothing
        objReader.Close()
    End Sub

    Public Sub SavCSV(ByVal Datei As String, ByVal Separator As Char, ByVal IncludeHeader As Boolean)
        Dim objReader As New StreamWriter(Datei)
        Dim i As Int32
        Dim hochkomma As Char = """"
        Dim zeile As String = ""
        Dim lItem As ListViewItem
        Dim celle As String

        'Erste Zeile mit Header hinzufgen
        If IncludeHeader = True Then
            objReader.WriteLine(ListviewHeader(Separator))
        End If

        For Each lItem In ListView1.Items
            zeile = ""
            For i = 0 To ListView1.Columns.Count - 1
                celle = lItem.SubItems.Item(i).Text
                If InStr(celle, Separator, CompareMethod.Text) > 0 Then
                    celle = hochkomma & celle & hochkomma
                End If
                zeile = zeile & celle
                If i < ListView1.Columns.Count - 1 Then
                    zeile = zeile & Separator
                End If
            Next
            objReader.WriteLine(zeile)
        Next
        objReader.Close()
    End Sub

    Private Function ListviewHeader(ByVal Separator As Char) As String
        Dim zeile As String = ""
        Dim i As Integer
        Dim header As String = "" '"﻿" 'Chr(0239) & Chr(0187) & Chr(0191)

        For i = 0 To ListView1.Columns.Count - 1
            zeile = zeile & ListView1.Columns.Item(i).Text & Separator
        Next
        ListviewHeader = header + zeile
    End Function

    Public Sub FullRowSelect(ByVal modus As Boolean)
        ListView1.FullRowSelect = modus
    End Sub

    Private Sub ZeileEinfarben()
        Dim InProgress As Boolean
        Dim lItem As ListViewItem

        For Each lItem In ListView1.Items
            If InProgress Then
                'dz Begin *********** BackColor property not supported in .NetCF 1.0
                'lItem.BackColor = Color.LightBlue
                'dz End ***********

                InProgress = False
            Else
                InProgress = True
            End If
        Next
    End Sub

    Public Sub Clear()
        Dim lItem As ListViewItem
        For Each lItem In ListView1.Items
            ListView1.Items.Remove(lItem)
        Next
    End Sub

    Public Sub Show()
        ListView1.Show()
    End Sub

    Public Sub Visible(ByVal modus As Boolean)
        ListView1.Visible = modus
    End Sub

    Public Property Left() As Int32
        Get
            Return ListView1.Left
        End Get
        Set(ByVal Value As Int32)
            ListView1.Left = Value
        End Set
    End Property

    Public Property Top() As Int32
        Get
            Return ListView1.Top
        End Get
        Set(ByVal Value As Int32)
            ListView1.Top = Value
        End Set
    End Property

    Public Property Width() As Int32
        Get
            Return ListView1.Width
        End Get
        Set(ByVal Value As Int32)
            ListView1.Width = Value
        End Set
    End Property

    Public Property Height() As Int32
        Get
            Return ListView1.Height
        End Get
        Set(ByVal Value As Int32)
            ListView1.Height = Value
        End Set
    End Property

    Public ReadOnly Property ItemCount() As Int32
        Get
            Return ListView1.Items.Count
        End Get
    End Property

    Public ReadOnly Property selecteditem() As Integer
        Get
            Return selectedRow
        End Get
    End Property

    Public ReadOnly Property DLLVersion() As Double
        Get
            Return 1.0
        End Get
    End Property

    Public Sub Dispose() Implements System.IDisposable.Dispose
        If Not ListView1 Is Nothing Then
            ListView1.Dispose()
        End If
    End Sub
End Class
